--[[
FlaskMinder Buff reminder utility inspired by Nytshade of TeAm TyPo

Initial code was based on AntiSlack by Battleguard of Balnazzar.

Author: DigitalSorceress
Date: 2009/04/21
Version: 0.0.03
]]

-- Some initialization of our happy local vars
local myVersion = "v0.0.03"


local soundList = {
  [1] =  { name = "none",        path = "" },
  [2] =  { name = "Raid Warn",   path = "Sound\\interface\\RaidWarning.wav" },
  [3] =  { name = "Moo",         path = "Sound\\Spells\\PolyMorphCow.wav" },
  [4] =  { name = "Moo2",        path = "Sound\\Creature\\Cow\\CowDeath.wav" },
  [5] =  { name = "Ta-Da",       path = "Sound\\Spells\\PVPFlagTaken.wav" },
  [6] =  { name = "Tadpole",     path = "Sound\\Creature\\BabyMurloc\\BabyMurlocA.wav" },
  [7] =  { name = "Yahh",        path = "Sound\\Character\\Human\\MaleNPC\\HumanMalePirateHeavy\\HumanMalePirateHeavyAggro.wav" },
  [8] =  { name = "Murmur",      path = "Sound\\Creature\\Murmur\\MurmurSpawn.wav" },
  [9] =  { name = "Turkey",      path = "Sound\\Creature\\turkey\\TurkeyAttackA.wav" },
  [10] = { name = "Klunk",       path = "Sound\\Doodad\\ArcaneCrystalOpen.wav" },
  [11] = { name = "Gong",        path = "Sound\\Doodad\\G_GongTroll01.wav" },
  [12] = { name = "Gong2",       path = "Sound\\Doodad\\BellTollHorde.wav" },
  [13] = { name = "Drum",        path = "Sound\\Doodad\\BellTollTribal.wav" },
  [14] = { name = "Ding Ding",   path = "Sound\\Doodad\\BoatDockedWarning.wav" },
  [15] = { name = "Boom",        path = "Sound\\Doodad\\Cannon01_BlastA.wav" },
  [16] = { name = "Klank",       path = "Sound\\Doodad\\DireMaulCrystalGeneratorOpen.wav" },
  [17] = { name = "Thunk",       path = "Sound\\Item\\Weapons\\Shields\\MetalShieldBlock2.wav" },
  [18] = { name = "USE DEFAULT", path = "" },
}
-- FlaskMinder = LibStub("AceAddon-3.0"):NewAddon("FlaskMinder", "AceConsole-3.0","AceComm-3.0","AceEvent-3.0","AceSerializer-3.0")
FlaskMinder = LibStub("AceAddon-3.0"):NewAddon("FlaskMinder", "AceConsole-3.0","AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("FlaskMinder")

-- MUST set these after the Localizations are pulled in 
local masterList = {}
masterList.GEx = L["GUARDIAN_LIST"];
masterList.BEx = L["BATTLE_LIST"];
masterList.FEx = L["FLASK_LIST"];
masterList.XEx = { -- Exclusion list: used for filtering buffs that have the same name as elixirs but are different effects
  "Interface\\Icons\\Ability_Warrior_DefensiveStance", -- Armor buff from scrolls (conflicts with Defense potions)
  "Interface\\Icons\\Spell_Frost_WindWalkOn", -- Shaman's Water Walking spell
  "Interface\\Icons\\Spell_Holy_BlessingOfAgility", -- Agility buff from scrolls (conflicts with Agility potions)
  "Interface\\Icons\\Spell_Nature_Strength", -- Strength buff from scrolls (conflicts with Strength potions)
  "Interface\\Icons\\Spell_Holy_MindVision", -- Trinket proc from Heavenly Inspiration (http://www.wowhead.com/?item=30293), conflicts with Healing Power potion
  "Interface\\Icons\\Ability_Warrior_ShieldMastery", -- ?? I don't remember why I put them here, but I must have had a good reason for that,
  "Interface\\Icons\\Spell_Nature_UnyeildingStamina", -- ?? I'll leave these here for now...
  "Interface\\Icons\\Spell_Nature_UnrelentingStorm" -- Proc from Mongoose enchant (http://www.wowhead.com/?spell=28093), conflicts with Haste potion
};

-- ------------------------------------------------
-- CONFIGURATION STUFF
-- ------------------------------------------------
-- Holds our configuration options
local options = {
  name = "FlaskMinder",
  handler = FlaskMinder,
  type = "group",
  childGroups = "select",
  get = "getConfigOption",
  set = "setConfigOption",
  args = {
    mainDescription = {
      type = "description",
      name = L["MAIN_DESCRIPTION"],
      order = 1
    },
    strictOptions = {
      type = "group",
      name = L["STRICT_GROUP_NAME"],
      inline = true,
      order = 2,
      args = {
        strictGroupDesc = {
          type = "description",
          name = L["STRICT_GROUP_DESC"],
          order = 1
        },
        strictFoodFlag = {
          type = "toggle",
          name = L["STRICT_FOOD_NAME"],
          desc = L["STRICT_FOOD_DESC"],
          order = 2
        },
        strictElixirFlag = {
          type = "toggle",
          name = L["STRICT_ELIXIR_NAME"],
          desc = L["STRICT_ELIXIR_DESC"],
          order = 3
        }
      }
    },
    updateOptions = {
      type = "group",
      name = L["UPDATE_GROUP_NAME"],
      inline = true,
      order = 3,
      args = {
        updateGroupDesc = {
          type = "description",
          name = L["UPDATE_GROUP_DESC"],
          order = 1
        },
        updateEnterCombat = {
          type = "toggle",
          name = L["UPDATE_ENTER_COMBAT_NAME"],
          desc = L["UPDATE_ENTER_COMBAT_DESC"],
          order = 2
        },
        updateExitCombat = {
          type = "toggle",
          name = L["UPDATE_EXIT_COMBAT_NAME"],
          desc = L["UPDATE_EXIT_COMBAT_DESC"],
          order = 3
        }
      }
    },
    defaultSoundGroupContainer = {
      type = "group",
      name = L["DEFAULT_SOUND_GROUP_NAME"],
      inline = true,
      order = 4,
      args = {
        defaultSoundGroupDesc = {
          type = "description",
          name = L["DEFAULT_SOUND_GROUP_DESC"],
          order = 1
        },
        defaultWarnSound = {
          type = "select",
          name = L["PICK_SOUND_NAME"],
          desc = L["PICK_SOUND_DESC"],
          order = 2,
          values = function(info) return FlaskMinder:getSoundSelectTable(true) end,
        },
        defaultWarnSoundTest = {
          type = "execute",
          name = L["PICK_SOUND_TEST"],
          func = function(info) FlaskMinder:soundCheck("defaultWarnSound") end,
          order = 3,
        }
      }
    },
  }
}


local debugOptions = {
  name = L["DEBUG_OPTIONS_NAME"],
  handler = FlaskMinder,
  type = "group",
  order = 1,
  get = "getConfigOption",
  set = "setConfigOption",
  args = {
    debugFlag = {
      type = "toggle",
      name = L["DEBUG_NAME"],
      desc = L["DEBUG_DESC"],
      order = 1,
    },
    blatherFlag = {
      type = "toggle",
      name = L["BLATHER_NAME"],
      desc = L["BLATHER_DESC"],
      order = 2,
    }
  }
}


--
-- Set the default profile if they don't have one
--
local defaults = {
  profile = {
    strictFoodFlag = false,
    strictElixirFlag = false,
    updateEnterCombat = false,
    updateExitCombat = true,
    debugFlag = false,
    blatherFlag = false,
    defaultWarnSound = 2,
  }
}


--
-- Field handlers
--
function FlaskMinder:getConfigOption(key)
  self:debugMsg("getConfigOption()", "debug")
  self:debugMsg("  key[#key]: "..tostring(key[#key]), "debug")
  return self.db.profile[key[#key]]
end
  

function FlaskMinder:setConfigOption(key, value)
  self:debugMsg("setConfigOption()", "debug")
  self:debugMsg("  key: "..tostring(key[#key]), "debug")
  self:debugMsg("  val: "..tostring(value), "debug")
  self.db.profile[key[#key]] = value
end


--
-- Chat Command setup
-- 
function FlaskMinder:ChatCommand(input)
  self:debugMsg("ChatCommand initiated", "debug");
  self:debugMsg("  input: "..tostring(input), "debug");
  
  if not input or input:trim() == "" then
    -- This would use the Ace config gui
    -- LibStub("AceConfigDialog-3.0"):Open("FlaskMinder")
    -- This uses the Bliz options pane instead
    InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
  elseif input == "debug" then
    local newValue = true;
    
    if self.db.profile.debugFlag then
      newValue = false;
    else
      newValue = true;
    end
    self.db.profile.debugFlag = newValue

  elseif input == "blather" then
    local newValue = true;
    
    if self.db.profile.blatherFlag then
      newValue = false;
    else
      newValue = true;
    end
    self.db.profile.blatherFlag = newValue

  elseif input == "check" or input == "checkNow" then

    self:checkBuffs( false );
  else
    --LibStub("AceConfigCmd-3.0").HandleCommand(FlaskMinder, L["SHORT_SLASH_CMD"], L["FULL_SLASH_CMD"], input)
    self:helpCommand(input);
  end
end

-- ------------------------------------------------
-- BASIC REQUIRED FUNCTIONS
-- ------------------------------------------------

-- Called when the addon is loaded
function FlaskMinder:OnInitialize()
  -- Using the AceDB for storage of our info
  self.db = LibStub("AceDB-3.0"):New("FlaskMinderDB", defaults, "Default")

  -- main options table stuff
  LibStub("AceConfig-3.0"):RegisterOptionsTable("FlaskMinder Options", options)
  LibStub("AceConfig-3.0"):RegisterOptionsTable("FlaskMinder Debug Options", debugOptions)
  LibStub("AceConfig-3.0"):RegisterOptionsTable("FlaskMinder Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db))

  -- adding to Blizzard options configuration
  self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("FlaskMinder Options", "FlaskMinder")
  self.optionsFrame.debugOptions = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("FlaskMinder Debug Options", L["DEBUG_OPTIONS_NAME"], "FlaskMinder")
  self.optionsFrame.Profiles = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("FlaskMinder Profiles", "Profiles", "FlaskMinder")

  -- registering the slash commands
  self:RegisterChatCommand(L["FULL_SLASH_CMD"], "ChatCommand")
  self:RegisterChatCommand(L["SHORT_SLASH_CMD"], "ChatCommand")
  -- self:RegisterChatCommand("help", "helpCommand")
end


-- Called when the addon is enabled
function FlaskMinder:OnEnable()
  -------------------
  -- REGISTER EVENTS
  -------------------
  self:RegisterEvent("PLAYER_REGEN_DISABLED", "enterCombat")
  self:RegisterEvent("PLAYER_REGEN_ENABLED", "exitCombat")
  
  
  -----------------------------
  -- SECUE HOOKS (post-hooking)
  -----------------------------
  -- NONE


  --------------------
  -- FINAL PRE-LAUNCH
  --------------------

  self:Print("FlaskMinder "..myVersion.." ENABLED!")
end


-- Called when the addon is disabled
function FlaskMinder:OnDisable()
  --
end


-- ------------------------------------------------
-- EVENT HANDLERS
-- ------------------------------------------------

-- Fired when player enters combat
function FlaskMinder:enterCombat(info)
  self:debugMsg("enterCombat()", "debug")
  self:debugMsg("  info: "..tostring(info), "blather")
  self:debugMsg("  updateEnterCombat: "..tostring(self.db.profile["updateEnterCombat"]), "blather")

  if self.db.profile["updateEnterCombat"] then
    self:debugMsg("  updateEnterCombat Triggered: ", "debug")
    -- Do buff check - ignore food warning since you can't fix it now
    self:checkBuffs( true )
  end
end

-- Fired when player leaves combat
function FlaskMinder:exitCombat(info)
  self:debugMsg("exitCombat()", "debug")
  self:debugMsg("  info: "..tostring(info), "blather")
  self:debugMsg("  exitCombatFlag: "..tostring(self.db.profile["exitCombatFlag"]), "blather")


  if self.db.profile["updateExitCombat"] then
    self:debugMsg("  updateExitCombat Triggered: ", "debug")
   -- Do buff check - Include food
    self:checkBuffs( false )
  end
end


-- ------------------------------------------------
-- MAIN APP FUNCTIONALITY
-- ------------------------------------------------
function FlaskMinder:checkBuffs(isInCombat)
  self:debugMsg("checkBuffs()", "debug");
  self:debugMsg("  info: "..tostring(info), "blather");
  self:debugMsg("  isInCombat: "..tostring(isInCombat), "blather");
  
  local niceName = "Your";
  local warnSoundPlayed = false; -- used to keep from spamming sounds
  local foodWarnMsg = "";
  local warnMsg = "";
  
  playerBuffs = self:getUnitBuffs( "player" );
  self:debugMsg("  playerBuffs: "..tostring(playerBuffs), "blather");

  if playerBuffs.name then
    niceName = tostring(playerBuffs.name).."'s";
    self:debugMsg("  niceName: "..tostring(niceName), "blather");
  end
  
  -- Handle food warnings
  self:debugMsg("Checking Food Buff", "debug");
  if self.db.profile.strictFoodFlag and not isInCombat then
    self:debugMsg("  strictFoodFlag set and not in combat", "debug");
    self:debugMsg("    playerBuffs table food info:", "blather");
    self:debugMsg("      foodName: ".. tostring(playerBuffs.foodName), "blather");
    self:debugMsg("      foodTimeLeft: ".. tostring(playerBuffs.foodTimeLeft), "blather");
    self:debugMsg("      foodFound: ".. tostring(playerBuffs.foodFound), "blather");
   
    self:debugMsg("    checking foodFound", "debug");
    if not playerBuffs.foodFound then
      self:debugMsg("    FAILED foodFound check", "debug");
      -- NOT FOUND AT ALL
      foodWarnMsg = niceName.." food buff MISSING";
      
      if not warnSoundPlayed then
        self:playSoundByIndex(self.db.profile.defaultWarnSound);
        warnSoundPlayed = true;
      end
      UIErrorsFrame:AddMessage(foodWarnMsg, 1.0, 0.0, 0.0, 5.0);
    else
      self:debugMsg("    PASSED foodFound, checking foodTimeLeft", "debug");
      if playerBuffs.foodFound and playerBuffs.foodTimeLeft < 300 then
        self:debugMsg("    FAILED foodTimeLeft check", "debug");
        
        foodWarnMsg = niceName.." food buff will expire in "..self:secToHumanTime( playerBuffs.foodTimeLeft );

        if not warnSoundPlayed then
          self:playSoundByIndex(self.db.profile.defaultWarnSound);
          warnSoundPlayed = true;
        end
        UIErrorsFrame:AddMessage(foodWarnMsg, 1.0, 1.0, 0.5, 5.0);
      end  -- foodTimeLeft
    end 
  end  -- food checks
  
  -- HANDLE FLASKS/ELIXIRS
  --
  -- FLASK FOUND SECTION
  --
  self:debugMsg("Checking Flasks and/or Elixirs", "debug");
  if playerBuffs.flaskFound then
    self:debugMsg("  in flaskFound section ", "debug");

    -- flask found, don't bother with elixirs at all, but DO check for flask warning
    self:debugMsg("  checking flaskTimeLeft", "debug");
    if playerBuffs.flaskTimeLeft < 300 then    
      self:debugMsg("    FAILED flaskTimeLeft check", "debug");
      warnMsg = niceName.." "..tostring(playerBuffs.flaskName).." will expire in "..self:secToHumanTime( playerBuffs.flaskTimeLeft );
      
      if not warnSoundPlayed then
        self:playSoundByIndex(self.db.profile.defaultWarnSound);
        warnSoundPlayed = true;
      end
      UIErrorsFrame:AddMessage(warnMsg, 1.0, 1.0, 0.5, 5.0);
    end 
  --
  -- NOTHING FOUND SECTION
  --
  elseif not playerBuffs.flaskFound and not playerBuffs.gExFound and not playerBuffs.bExFound then
    self:debugMsg("  in not flaskFound AND not gExFound AND not bExFound section", "debug");

    -- ABSOLUTELY NOTHING found, SCREAM
    warnMsg = tostring(playerBuffs.name).." Has no flask or elixirs";
    
    if not warnSoundPlayed then
      self:playSoundByIndex(self.db.profile.defaultWarnSound);
      warnSoundPlayed = true;
    end
    UIErrorsFrame:AddMessage(warnMsg, 1.0, 0.0, 0.0, 5.0);
  --
  -- STRICT ELIXIRS SECTION
  --
  elseif self.db.profile.strictElixirFlag then
    self:debugMsg("  in strictElixirFlag section", "debug");

    -- We have no flask, but we may have a guardian or battle elixir
    --   SCREAM if we have nothing
    --   also, throw out warnings for whichever they have

    --
    -- STRICT GUARDIAN ELIXIR SECTION
    --
    self:debugMsg("    checking gExFound", "debug");
    if not playerBuffs.gExFound then
      self:debugMsg("    FAILED gExFound", "debug");

      warnMsg = tostring(playerBuffs.name).." Has no GUARDIAN elixir";

      if not warnSoundPlayed then
        self:playSoundByIndex(self.db.profile.defaultWarnSound);
        warnSoundPlayed = true;
      end
      UIErrorsFrame:AddMessage(warnMsg, 1.0, 1.0, 0.5, 5.0);
    elseif playerBuffs.gExFound and playerBuffs.gExTimeLeft < 300 then
      self:debugMsg("    PASSED gExFound, but gExTimeLeft below threshhold", "debug");
      -- warn the guardian elixir is running out
      warnMsg = niceName.." "..tostring(playerBuffs.gExName).." will expire in "..self:secToHumanTime( playerBuffs.gExTimeLeft );
      
      if not warnSoundPlayed then
        self:playSoundByIndex(self.db.profile.defaultWarnSound);
        warnSoundPlayed = true;
      end
      UIErrorsFrame:AddMessage(warnMsg, 1.0, 1.0, 0.5, 5.0);
    end

    --
    -- STRICT BATTLE ELIXIR SECTION
    --
    self:debugMsg("    checking bExFound", "debug");
    if not playerBuffs.bExFound then
      self:debugMsg("    FAILED bExFound", "debug");
      warnMsg = tostring(playerBuffs.name).." Has no BATTLE elixir";

      if not warnSoundPlayed then
        self:playSoundByIndex(self.db.profile.defaultWarnSound);
        warnSoundPlayed = true;
      end
      UIErrorsFrame:AddMessage(warnMsg, 1.0, 1.0, 0.5, 5.0);
    elseif playerBuffs.bExFound and playerBuffs.bExTimeLeft < 300 then
      self:debugMsg("    PASSED bExFound, but gExTimeLeft below threshhold", "debug");
      -- warn the guardian elixir is running out
      warnMsg = niceName.." "..tostring(playerBuffs.bExName).." will expire in "..self:secToHumanTime( playerBuffs.bExTimeLeft );
      
      if not warnSoundPlayed then
        self:playSoundByIndex(self.db.profile.defaultWarnSound);
        warnSoundPlayed = true;
      end
      UIErrorsFrame:AddMessage(warnMsg, 1.0, 1.0, 0.5, 5.0);
    end
  else
    self:debugMsg("  in unstrict elixir checks section", "debug");
    -- UNSTRICT checks

    self:debugMsg("    checking gExFound AND bExFound", "debug");
    if not playerBuffs.gExFound and not playerBuffs.bExFound then
      self:debugMsg("      FAILED check, both missing", "debug");
      -- if we got this far, we know we have no flask, and now, we know we have no elixirs either

      -- ABSOLUTELY NOTHING found, SCREAM
      warnMsg = tostring(playerBuffs.name).." Has no flask or elixirs";
      
      if not warnSoundPlayed then
        self:playSoundByIndex(self.db.profile.defaultWarnSound);
        warnSoundPlayed = true;
      end
      UIErrorsFrame:AddMessage(warnMsg, 1.0, 0.0, 0.0, 5.0);
      
    else
      self:debugMsg("      PASSED check, at least one elixir present... checking expirations", "debug");

      -- ok, something's up... make sure its not low
      
      -- UNSTRICT GUARDIAN ELIXIR (only warn if its there but low)
      self:debugMsg("    checking gExTimeLeft", "debug");
      if playerBuffs.gExFound and playerBuffs.gExTimeLeft < 300 then
        self:debugMsg("    FAILED gExTimeLeft check", "debug");
        -- warn the guardian elixir is running out
        warnMsg = niceName.." "..tostring(playerBuffs.gExName).." will expire in "..self:secToHumanTime( playerBuffs.gExTimeLeft );
        
        if not warnSoundPlayed then
          self:playSoundByIndex(self.db.profile.defaultWarnSound);
          warnSoundPlayed = true;
        end
        UIErrorsFrame:AddMessage(warnMsg, 1.0, 1.0, 0.5, 5.0);
      end
  
      -- UNSTRICT BATTLE ELIXIR (only warn if its there but low)
      self:debugMsg("    checking bExTimeLeft", "debug");
      if playerBuffs.bExFound and playerBuffs.bExTimeLeft < 300 then
        self:debugMsg("    FAILED bExTimeLeft check", "debug");
        -- warn the guardian elixir is running out
        warnMsg = niceName.." "..tostring(playerBuffs.bExName).." will expire in "..self:secToHumanTime( playerBuffs.bExTimeLeft );
        
        if not warnSoundPlayed then
          self:playSoundByIndex(self.db.profile.defaultWarnSound);
          warnSoundPlayed = true;
        end
        UIErrorsFrame:AddMessage(warnMsg, 1.0, 1.0, 0.5, 5.0);
      end
      
    end -- end if neither elixir else if low elixir

  end 
  warnSoundPlayed = false
end -- end of checkBuffs



--[[
  Much of the hard work was already done in AntiSlack by Battleguard of Balnazzar.

  BuffCheck()
  
  Checks the current UID for Food and Flask / Elixirs and returns table:
  
  returnTable = {
  ["UID"] = string,
  ["dead"] = bool,
  ["name"] = string,
  ["class"] = string,
  ["r"] = int,
  ["g"] = int,
  ["b"] = int,

  ["foodFound"] = bool,
  ["foodName"] = string,
  ["foodTimeLeft"] = int (sec),

  ["flaskFound"] = bool,
  ["flaskName"] = string,
  ["flaskTimeLeft"] = int (sec),
  
  ["gExFound"] = bool,
  ["gExName"] = string,
  ["gExTimeLeft"] = int (sec),
  
  ["bExFound"] = bool,
  ["bExName"] = string,
  ["bExTimeLeft"] = int (sec),  
  }
  
]]--
function FlaskMinder:getUnitBuffs(info, UID)
  
  -- if called without UID, then we assume it's self
  if UID == nil then
    UID = "player";
  end

  local returnTable = {};
  -- Initializing the returnTable values
  returnTable.UID = UID;
  returnTable.dead = UnitIsDead(UID);
  returnTable.name = UnitName(UID);
  returnTable.class = ({UnitClass(UID)})[2];
  returnTable.r = RAID_CLASS_COLORS[returnTable.class].r;
  returnTable.g = RAID_CLASS_COLORS[returnTable.class].g;
  returnTable.b = RAID_CLASS_COLORS[returnTable.class].b;

  returnTable.foodFound = false;
  returnTable.foodName = "NONE";
  returnTable.foodTimeLeft = 0;
  returnTable.flaskFound = false;
  returnTable.flaskName = "NONE";
  returnTable.flaskTimeLeft = 0;
  returnTable.gExFound = false;
  returnTable.gExName = "NONE";
  returnTable.gExTimeLeft = 0;
  returnTable.bExFound = false;
  returnTable.bExName = "NONE";
  returnTable.bExTimeLeft = 0;

  -- Iterate through the buffs and get time left / existence on flasks, elixirs, etc...
  for j = 1, 40 do
    local buffname, _, bufficon, _, _, _, timeleft = UnitBuff(UID, j);

    if buffname then
      if not returnTable.foodFound and buffname == L["WELL_FED"] then
        returnTable.foodName = L["WELL_FED"];
        returnTable.foodTimeLeft = self:expTimeToSeconds( timeleft );
        returnTable.foodFound = true;
      elseif not returnTable.flaskFound and self:isStringInTable(buffname, masterList.FEx) then
        returnTable.flaskName = buffname;
        returnTable.flaskTimeLeft = self:expTimeToSeconds( timeleft );
        returnTable.flaskFound = true;
        -- Clearing the other values cuz flask is king    
        returnTable.gExFound = false;
        returnTable.gExName = "NONE";
        returnTable.gExTimeLeft = 0;
        returnTable.bExFound = false;
        returnTable.bExName = "NONE";
        returnTable.bExTimeLeft = 0;
      elseif not returnTable.flaskFound and not returnTable.gExFound and self:isStringInTable(buffname, masterList.GEx) and not self:isStringInTable(bufficon, masterList.XEx) then
        returnTable.gExName = buffname;
        returnTable.gExTimeLeft = self:expTimeToSeconds( timeleft );
        returnTable.gExFound = true;
      elseif not returnTable.flaskFound and not returnTable.bExFound and self:isStringInTable(buffname, masterList.BEx) and not self:isStringInTable(bufficon, masterList.XEx) then
        returnTable.bExName = buffname;
        returnTable.bExTimeLeft = self:expTimeToSeconds( timeleft );
        returnTable.bExFound = true;
      elseif returnTable.foodFound and ( flaskFound or ( gExFound and bExFound ) ) then
        -- stop looking, we have all that we're after
        break;
      end
    else
      break;
    end
  end
  
  -- Always sending back a valid returnTable
  return returnTable;
end




-- ------------------------------------------------
-- EVENT HANDLERS
-- ------------------------------------------------


-- ------------------------------------------------
-- HOOKED FUNCTIONS
-- ------------------------------------------------


-- ------------------------------------------------
-- COMMUNICATIONS HANDLER
-- ------------------------------------------------

-- --
-- -- Handles received commands
-- --
-- -- NOTE: This should REALLY get broken up into separate items / tableize
-- --
-- function FlaskMinder:OnCommReceived(prefix, message, distribution, target)
--   -- Deserialize incoming message
--   local success, o = self:Deserialize(message)
--   if (self.db.profile.debugFlag) then
--     self:debugMsg("OnCommReceived():: ENGAGED", "debug")
--     self:debugMsg("  prefix:"..tostring(prefix), "blather")
--     self:debugMsg("  message:"..tostring(message), "blather")
--     self:debugMsg("  distribution:"..tostring(distribution), "blather")
--     self:debugMsg("  target:"..tostring(target), "blather")
--     self:debugMsg("  player:"..UnitName("player"), "blather")
--     self:debugMsg("  Deserialize success:"..tostring(success), "blather")
--     self:debugMsg("  Message Content:"..tostring(o), "blather")
--   end -- end if debugFlag
-- 
--   -- don't do anyting if we could not deserialize
--   if (success == false) then
--     self:debugMsg("Failed to deserialize the message - OnCommReceived can not process any further", "error")
--     return -- Failure
--   else
--     --
--     -- TEMPLATE - Not actually used
--     --
--     if (o["op"] == "foo") then
--       if (self.db.profile.actionFlag) then
--         self:debugMsg("FOO op ENGAGED", "debug")
-- 
--         -- Put any extra conditions here
--         if ("someCondition" == "some value") then
--           self:debugMsg("debug_text_here", "blather")
--           return -- do nothing
--         else
--           -- do the action
--           self:debugMsg("debug_text_here", "blather")
-- 
--           -- actual command such as TakeTaxiNode(o["node"], "noComm")
--         end -- ending extra conditions
--       end -- if gossipFlag
--     end -- end if op == foo
--     --
--     -- END TEMPLATE
--     --
-- 
--   end -- if success==false
-- end -- function OnCommReceived


-- --
-- -- Wraps up Remote procedure call commands with a "nocomm" throttler
-- --
-- function FlaskMinder:rpcCommandSender(prefix, text, distribution, target, prio, noComm)
--   --debug
--   if (self.db.profile.debugFlag) then
--     self:debugMsg("rpcCommandSender()", "debug")
--     self:debugMsg("  prefix: "..tostring(prefix), "blather")
--     self:debugMsg("  text: "..tostring(text), "blather")
--     self:debugMsg("  distribution: "..tostring(distribution), "blather")
--     self:debugMsg("  target: "..tostring(target), "blather")
--     self:debugMsg("  prio: "..tostring(prio), "blather")
--     self:debugMsg("  noComm flag: "..tostring(noComm), "blather")
--   end
-- 
--   -- New throttling - DO NOT send msg unless noComm is nil
--   if (noComm) then
--     self:debugMsg("noComm flag IS set, NOT sending command", "blather")
--     -- Teh Gogglez - They do NOTHING!
--     return
--   else
--     self:debugMsg("noComm flag NOT set, sending command", "blather")
--     if (GetNumPartyMembers() == 0 and (distribution == "RAID" or distribution == "PARTY")) then
--       -- NOT in party, but trying to send to party - modify distirbution and target
--       distribution = "WHISPER"
--       target = GetUnitName("player")
--     end
--     self:SendCommMessage(prefix, text, distribution, target, prio)     
--   end
-- end


-- ------------------------------------------------
-- UTILITY FUNCTIONS
-- ------------------------------------------------
function FlaskMinder:helpCommand( input )

  self:Print( L["HELP_MENU"] );
end


function FlaskMinder:print_r( targetTable )
  self:debugMsg("DUMPING contents of table"..tostring(targetTable), "blather");
  local returnString = "";
  for k,v in ipairs( targetTable ) do
    self:debugMsg("  Found Key: "..tostring(k), "blather");
    self:debugMsg("  Found Value: "..tostring(v), "blather");
    returnString = returnString.."["..tostring(k).."] = "..tostring(v).."\n";
  end
  return returnString;
end


--[[
  secToHumanTime is a simple funciton to take a value in seconds and return 
  a more human readable time format.
  
  Depending on the time left, we will return one of the three 
  (where X is number of hours, Y is number of minites, and Z is num of seconds)
    "Xh Ym"
    "Ym"  
    "Zs"
]]--
function FlaskMinder:secToHumanTime( seconds )
  local returnVal = "0s"
  local hrs = 0
  local mins = 0
  local secs = 0
  
  if seconds then  -- avoids nil errors
    if seconds < 60 then -- less than a minute, break into seconds
      secs = seconds
    elseif seconds < 3600 then -- less than an hour (but more than a minute) use minutes
      mins = math.floor( 1 + ( seconds / 60 ) )
    else -- more than an hour use hours/mins
      -- get raw hours
      hrs = math.floor( seconds / 3600 )
      -- get raw mins
      mins = math.floor( 1 + ( ( seconds - ( hrs * 3600 ) ) / 60 ) )
    end
  end

  -- format up the results nicely for return
  if hrs > 0 then
    returnVal = string.format( "%u h, %u m", hrs, mins )
  elseif mins > 0 then
    returnVal = string.format( "%u m", mins, secs )
  else
    returnVal = string.format( "%u s", secs )
  end      

  return returnVal
end


function FlaskMinder:expTimeToSeconds( timeValue )
  return math.floor(-1*(GetTime()-timeValue))
end


-- Checks if a string is in a table
-- Borrowed from Battleguard's AntiSlack
function FlaskMinder:isStringInTable( targetString, targetTable )
  local stringFound = false 
  
  -- loop through all values
  for i = 1, #targetTable do
    if targetTable[i] == targetString then
      stringFound = true
      break
    end
  end

  return stringFound
end

------------------
-- Sound handling
------------------

--
-- Plays a chosen sound from soundList by its FlaskMinder option
--
function FlaskMinder:soundCheck(key)
  self:debugMsg("soundCheck()", "debug")
  self:debugMsg("  key: "..tostring(key), "debug")
  -- need nil check to keep uglyness from happening
  if (key ~= nil) then
    self:playSoundByIndex(self.db.profile[key])
  end
end

--
-- Plays a chosen sound from soundList by its index (in soundList)
--
function FlaskMinder:playSoundByIndex(index)
  self:debugMsg("playSoundByIndex()", "debug")
  self:debugMsg("  index:"..tostring(index), "blather")
  -- check if USE DEFAULT
  if (index == #soundList) then
    self:debugMsg("  USE DEFAULT detected, altering choice...", "blather")
    -- alter the index to the default
    index = self.db.profile.defaultWarnSound
    self:debugMsg("    new index:"..tostring(index), "blather")
  end
  
  -- 1 will always be NONE, so only play if it's something else
  if (index > 1) then
    PlaySoundFile(self:getSoundPathByIndex(index))
    self:debugMsg("Playing sound "..tostring(self:getSoundNameByIndex(index)), "debug")
  else
    self:debugMsg("  Not playing sound due to user selection", "blather")
  end
end

--
-- Returns the name of the item defined in soundList given the soundList index
--
function FlaskMinder:getSoundNameByIndex(index)
  return soundList[index].name
end

--
-- Returns the path of the item defined in soundList given the soundList index
--
function FlaskMinder:getSoundPathByIndex(index)
  return soundList[index].path
end

--
-- Wrapper for soundList to make the type of array needed by AceConfig select
--
-- NOTE: If passed any non-false value in skipDefault, then it removes the last
--       entry which should always be "USE DEFAULT". This is needed for the 
--       config section of the default sound so it doesn't end up referring 
--       to itself.
function FlaskMinder:getSoundSelectTable(skipDefault)
  self:debugMsg("getSoundSelectTable()", "debug")
  self:debugMsg("  skipDefault:"..tostring(skipDefault), "blather")
  
  local returnTable = {}
  -- turn the soundList into a flat table
  for k,v in ipairs(soundList) do
    --[[ -- not used
    self:debugMsg("adding to returnTable", "debug")
    self:debugMsg("  key: "..tostring(k), "debug")
    self:debugMsg("  val: "..tostring(v), "debug")
    self:debugMsg("  val.name: "..tostring(v.name), "debug")
    self:debugMsg("  val.path: "..tostring(v.path), "debug")
    ]]
    table.insert(returnTable, k, v.name)
  end
  
  -- Handle removing the last entry (should always be "USE DEFAULT")
  if (skipDefault ~= nil and skipDefault ~= false) then
    self:debugMsg("  skipDefault set... removing index: "..tostring(#returnTable), "blather")
    -- we don't want the default sound to be able to use "USE DEFAULT"
    table.remove(returnTable, #returnTable)
  end
  
  -- Pass back the resulting table we created
  return returnTable
end


--
-- Takes a raw Amount of money (WOW always just passes costs in copper and 
-- returns either formatted text "Xg Xs Xc" or three component values g,s,c
--
-- @param int rawAmount The amount of money (in copper) to parse
-- @param string outputType Controls the form of output (currently only text or default)
-- @return string  (if outputType == "text")
-- @return int,int,int  g,s,c  Values for Gold, Silver, and Copper respectively
function FlaskMinder:copperToCurrency(rawAmount, outputType)
  -- calculate amounts
  -- raw amount is copper - 1g = 10,000c, so just get raw/10,000 and discard remainder
  local g = math.floor( rawAmount / 10000 )
  -- raw amount is copper, 1s = 100 c... subtract g*10,000 copper and do division, discard remainder
  local s = math.floor( ( rawAmount - ( g * 10000 ) ) / 100 )
  -- by doing modulus of rawAmount and 100, we just get copper left
  local c = rawAmount % 100

  -- format return
  if (outputType == "text") then
    -- output will be string formatted as  Xg Xs Xc
    -- return tostring(g).."g "..tostring(s).."s "..tostring(c).."c"
    return string.format("%ug %us %uc", g,s,c)
  else
    -- three separate values returned by default... gold, silver, copper
    return g,s,c
  end
end


--
-- Debugging and console message handling
--
function FlaskMinder:debugMsg(text, level)
  if (level == nil) then
    return -- do nothing
  elseif (level == "error") then
    PlaySound("RaidWarning");
    UIErrorsFrame:AddMessage(text, 1.0, 0.0, 0.0, 5.0)
  elseif (level == "warn") then   
    PlaySoundFile(self:getSoundPathByIndex(self.db.profile.defaultWarnSound))
    UIErrorsFrame:AddMessage(text, 1.0, 1.0, 0.5, 5.0)
  elseif (level == "notice") then
    self:Print(level..": "..text)
  elseif (level == "debug" and (self.db.profile.debugFlag or self.db.profile.blatherFlag)) then
    self:Print(level..": "..text)
  elseif (level == "blather" and self.db.profile.blatherFlag) then
    self:Print(level..": "..text)
  end
end
